Package: text
Type: Package
Title: Analyses of Text using Transformers Models from HuggingFace,
        Natural Language Processing and Machine Learning
Version: 1.9
Authors@R: c(
    person("Oscar", "Kjell", , "oscar.kjell@psy.lu.se", role = c("aut", "cre"), 
      comment = c(ORCID = "0000-0002-2728-6278"),
    ),
    person("Salvatore", "Giorgi", role = c("aut"), 
      comment = c(ORCID = "0000-0001-7381-6295")
    ),
    person("Andrew", "Schwartz", role = c("aut"), 
      comment = c(ORCID = "0000-0002-6383-3339"))
    )
Description: Link R with Transformers from Hugging Face to transform text variables to word embeddings; where the word embeddings are used to statistically test the mean difference between set of texts, compute semantic similarity scores between texts, predict numerical variables, and visual statistically significant words according to various dimensions etc. For more information see  <https://www.r-text.org>.
License: GPL-3
URL: https://r-text.org/, https://github.com/OscarKjell/text/
BugReports: https://github.com/OscarKjell/text/issues/
Encoding: UTF-8
Archs: x64
SystemRequirements: Python (>= 3.6.0)
LazyData: true
BuildVignettes: true
Imports: topics, dplyr, tibble, stringi, tidyr, ggplot2, ggrepel,
        cowplot, rlang, purrr, magrittr, parsnip, recipes (>= 0.1.16),
        rsample, reticulate, tune, workflows, yardstick, future, furrr,
        hardhat
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, testthat, rio, glmnet, randomForest,
        overlapping, covr, xml2, ranger, utils, ggwordcloud, reactable,
        osfr, vdiffr, svglite
VignetteBuilder: knitr
Depends: R (>= 4.1.0)
NeedsCompilation: no
Packaged: 2026-06-13 07:46:06 UTC; oscarkjell
Author: Oscar Kjell [aut, cre] (ORCID: <https://orcid.org/0000-0002-2728-6278>),
  Salvatore Giorgi [aut] (ORCID: <https://orcid.org/0000-0001-7381-6295>),
  Andrew Schwartz [aut] (ORCID: <https://orcid.org/0000-0002-6383-3339>)
Maintainer: Oscar Kjell <oscar.kjell@psy.lu.se>
Repository: CRAN
Date/Publication: 2026-06-13 16:20:02 UTC
