# Generated by roxygen2: do not edit by hand

export(find_textrpp_env)
export(textAssess)
export(textBERTopics)
export(textCentrality)
export(textCentralityPlot)
export(textClassify)
export(textClean)
export(textCleanNonASCII)
export(textDescriptives)
export(textDiagnostics)
export(textDimName)
export(textDistance)
export(textDistanceMatrix)
export(textDistanceNorm)
export(textDomainCompare)
export(textEmbed)
export(textEmbedLayerAggregation)
export(textEmbedRawLayers)
export(textEmbedReduce)
export(textEmbedStatic)
export(textExamples)
export(textFindNonASCII)
export(textFineTuneDomain)
export(textFineTuneTask)
export(textGeneration)
export(textLBAM)
export(textModelLayers)
export(textModels)
export(textModelsRemove)
export(textNER)
export(textPCA)
export(textPCAPlot)
export(textPlot)
export(textPredict)
export(textPredictAll)
export(textPredictTest)
export(textProjection)
export(textProjectionPlot)
export(textQA)
export(textSimilarity)
export(textSimilarityMatrix)
export(textSimilarityNorm)
export(textSum)
export(textTokenize)
export(textTokenizeAndCount)
export(textTopics)
export(textTopicsReduce)
export(textTopicsTest)
export(textTopicsTree)
export(textTopicsWordcloud)
export(textTrain)
export(textTrainLists)
export(textTrainN)
export(textTrainNPlot)
export(textTrainRandomForest)
export(textTrainRegression)
export(textTranslate)
export(textWordPrediction)
export(textZeroShot)
export(textrpp_initialize)
export(textrpp_install)
export(textrpp_install_virtualenv)
export(textrpp_uninstall)
importFrom(cowplot,draw_plot)
importFrom(cowplot,ggdraw)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,add_row)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,between)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,desc)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,group_split)
importFrom(dplyr,left_join)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_if)
importFrom(dplyr,n)
importFrom(dplyr,ntile)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,select_if)
importFrom(dplyr,slice)
importFrom(dplyr,slice_head)
importFrom(dplyr,starts_with)
importFrom(dplyr,summarize)
importFrom(dplyr,transmute)
importFrom(furrr,future_map)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(ggplot2,aes)
importFrom(ggplot2,autoplot)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_color_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(hardhat,importance_weights)
importFrom(magrittr,"%>%")
importFrom(magrittr,set_colnames)
importFrom(parsnip,fit)
importFrom(parsnip,linear_reg)
importFrom(parsnip,logistic_reg)
importFrom(parsnip,multinom_reg)
importFrom(parsnip,rand_forest)
importFrom(parsnip,set_engine)
importFrom(purrr,as_vector)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map2_dfr)
importFrom(purrr,map_chr)
importFrom(purrr,map_lgl)
importFrom(purrr,reduce)
importFrom(recipes,all_numeric)
importFrom(recipes,all_predictors)
importFrom(recipes,bake)
importFrom(recipes,juice)
importFrom(recipes,prep)
importFrom(recipes,recipe)
importFrom(recipes,step_center)
importFrom(recipes,step_impute_knn)
importFrom(recipes,step_naomit)
importFrom(recipes,step_pca)
importFrom(recipes,step_scale)
importFrom(recipes,update_role)
importFrom(reticulate,source_python)
importFrom(rlang,.data)
importFrom(rlang,sym)
importFrom(rsample,analysis)
importFrom(rsample,assessment)
importFrom(rsample,bootstraps)
importFrom(rsample,initial_validation_split)
importFrom(rsample,validation_set)
importFrom(rsample,vfold_cv)
importFrom(stats,chisq.test)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,fisher.test)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,predict)
importFrom(stats,pt)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(stats,weighted.mean)
importFrom(stringi,stri_c)
importFrom(stringi,stri_count_regex)
importFrom(stringi,stri_count_words)
importFrom(stringi,stri_detect_fixed)
importFrom(stringi,stri_match)
importFrom(stringi,stri_split_boundaries)
importFrom(stringi,stri_split_regex)
importFrom(stringi,stri_trans_tolower)
importFrom(stringi,stri_trim)
importFrom(tibble,as_tibble)
importFrom(tibble,as_tibble_col)
importFrom(tibble,as_tibble_row)
importFrom(tibble,is_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,gather)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,separate)
importFrom(tidyr,uncount)
importFrom(tidyr,unnest)
importFrom(topics,topicsPlot)
importFrom(topics,topicsScatterLegend)
importFrom(topics,topicsTest)
importFrom(tune,collect_predictions)
importFrom(tune,control_grid)
importFrom(tune,control_resamples)
importFrom(tune,select_best)
importFrom(tune,tune)
importFrom(tune,tune_grid)
importFrom(utils,download.file)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
importFrom(utils,sessionInfo)
importFrom(workflows,add_case_weights)
importFrom(workflows,add_model)
importFrom(workflows,add_recipe)
importFrom(workflows,workflow)
importFrom(yardstick,accuracy)
importFrom(yardstick,bal_accuracy)
importFrom(yardstick,f_meas)
importFrom(yardstick,kap)
importFrom(yardstick,precision)
importFrom(yardstick,rmse)
importFrom(yardstick,roc_auc)
importFrom(yardstick,roc_auc_vec)
importFrom(yardstick,rsq)
importFrom(yardstick,sens)
importFrom(yardstick,spec)
