## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----timing_Computer_Capacity, eval = FALSE, warning=FALSE, message=FALSE-----
# 
# library(text)
# # Save starting time
# T1 <- Sys.time()
# textEmbed(Language_based_assessment_data_8_10[1,1],
#           layers = 12,
#           decontexts = FALSE)
# # Save stoping time
# T2 <- Sys.time()
# 
# # Compute time taken to run above function
# T2-T1

