## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup, eval = FALSE, warning=FALSE, message=FALSE------------------------
# 
# library(text)
# 
# # Check downloaded, available models.
# textModels()
# 
# # Delete a specified model and model associated files.
# textModelsRemove("name_of_model_to_be_removed")
# 

