% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4_2_textPlotPCA.R
\name{textPCA}
\alias{textPCA}
\title{textPCA()}
\usage{
textPCA(
  words,
  word_types_embeddings = word_types_embeddings_df,
  to_lower_case = TRUE,
  seed = 1010
)
}
\arguments{
\item{words}{Word or text variable to be plotted.}

\item{word_types_embeddings}{Word embeddings from textEmbed for individual words
(i.e., decontextualized embeddings).}

\item{to_lower_case}{Lower case words}

\item{seed}{Set different seed.}
}
\value{
A dataframe with words, their frequency and two PCA dimensions from the word_embeddings
for the individual words that is used for the plotting in the textPCAPlot function.
}
\description{
textPCA() computes 2 PCA dimensions of the word embeddings for individual words.
}
\examples{
\dontrun{
# Data
df_for_plotting2d <- textPCA(
  words = Language_based_assessment_data_8$harmonywords,
  word_types_embeddings = word_embeddings_4$word_types
)
df_for_plotting2d
}
}
\seealso{
see \code{\link{textPCAPlot}}
}
